__code__='Gas1]?ZXsi&Dr[^a)OI]gee!X<4!A!#NW7C%`9t/*B/\\t-mhm%%AUdR\'^[2D,>7=dS2cu0.@oFO0>c?!A*B1r_%XG<eX2Z%kAKI1\\,6\'iY3pL`_qa]!bo^W]J!%PdhuWb3!YF99YWXkZ\'-IIK!\\^FcK/<RBR",r["0`*A!8%?X"De0H!*fSQ#B1RM,%(ZF?,/)N-,:82pAciK^(9u>i.lh)!IbH%"n4#"-\\0g)jl/([\\aKE>"b%b8$bQ\\b->+GQO:=0,5s5/L[\\$amPh9ig:H;F9EjuUShIi\\;%d+3VlN4Up=VX56GuT?^eaZOfBCaqWcVfA-L7pU8[V.R4%N*"C%LG*,E;>1o)W6:\'aWQgHrY&sBn!I+@g40h=ZFdNFi+APf&4tDT@N-(!r0t73p$ffedDilsO*YEKYU=Rd[43!`fqOV..+(2YZuR&QQ6Id&EPCTR43=+f>*RK#X"\'E6Xb&4XSTSSIT<#1\\gZ_LU\'^,G::)]mk*LB6]EG7FgJS9tEk_R+0G-[(r>J?^4A*Nl%g\\8`;:1@N_q>TtW&>,Ml2T8[lF(UODpW[)Z2G:nCmuBlVVYs>\'WR>[g@rJo=7s:-OJK77S9QGu5VMHMIG.Kcu\\$\'rJD81RBFQBB81CeSu0HE55@1M,1ff0E"8YDh8Yu>5WBbi/7$IeA:^HbI`5o\'X7`TeRGC6IhMr9aK9_QNjplF0pOGcebKWH-9XVl?dD;<]mg9t.VrQhV!^%8a\'DX11;`\'-0nVcAu&aBLe+3ltiE5r?.F:"`=g`jkFR[enEjLp%;C1f"^1ga$o_Bj[#W;8qNECgA#5UHle:CjICsgE+f+UEdP5?R"8I`_L3:C7*/ck6/$K@MJh7rqE*KV$/HWW032D%RKMs5+GqIcd*G.m5"Z@\\SieI5,8j6UUlXmj;&WLfHDP&a=Nt$$%[lPt/?!3UH6QYqYft2>MI_\\aoL-&p]+UMKc;`\'_)l$6Y/S8-9^8O1lMWnnX\\SV,-iiY&1<Me%SWl+9l:K0lu<k7/c5o^P;KkrJ]fnd!TkG:-d`M$b/<C-Y_QEp2>RD-Dun9gWs\'UP8H_X->m75U!R!@-i>e,~>'



__code__='GatU0D/[on&>$QoP+t8\\;Ta?pcr<Nch??dg65:siJKd,+kRL.1\'NpH/PF2,]Zb)Q/V!n[9^@7ciCRnV_OFRHH^"XR2GA$&`B%Aa]\'Yep^J\'^%@8Dkd2P,*(8:r$js<ri7/"Xq2;>"me\'WW:JG!nIA;J.9:=J5_!IJ5*hSJDU>J"$@\'<!cSA5>JOCWPCR_)hZ9;Mr#Y>ai#0>:JEHpY"\'tu?#WPU.-_=BWle\\u>n8AJb2&On124/rpJZ?Vg.YnmZ@I7b<SeAVFQ9ukc!t8?t;@HTf=:<=I.Q$U7HugQtMh*(I&75e^<hV`kYsqhA;(H7FeK8`k"RHLUos+o?@%ToR]b:Fq"^<F"WfGMV"gZmZ*[t+I_4QS-AWG+n7nNOb`aA*tloeOdf4$.TKZ\'anJ[O4_Bmi4%NH2Y.fO_H\\HE;/cRQH?b>a1(Vg*<q<).>[BEPP<dQ9&%[X+OcGg.UhPNmV:5f]e?3\'V!Bm4cQ_;m5aJ`@p`i*X[Q14MAagb+aM9*P@126h_s<6.)g2MYH6)D\'@i^-Oh[&H^Uu)7U*3cPG6S/U6;Q99=fSS?XNOW4nN]Q5"u;3*Po,31e&3T?1>Op_[q0E"X4,bXh/-r:)$fKp0HIQ4m^_MK2EdCNJp89f-r"6PV4(qW-BDT(m2^AjY[gq\\9g\':r2q<5EfojDmXh$tCYkjOVMRcE=Nrn39g+ZTtVG\\pr[I4Q*4EKb+5R#"P!."LCk*feDfRS#:6)jNKb,u;Gbf2^!$B/`_jsrTAS=\\=JY:e85EV90H2jS=s7utp]+m\':KX)num3RQs\']B1JE6mFnq!6jN:,*RS?`A2Yjoumg\\l\';[cUL^^TG^ej.41t,"au`i^3M+j9mn3WY/[5u"&O`n@_SKTVGsEP/0##"HcL4#P@ia5cq\'8RBQWVjC\\GYW\'YP<giA:][=\\Y-rb(1+R15]r@d")8#Zg^d^J@EgY4[dLO?`dRSMM.%*%hs^01dr70.q:_Q\\Yg6!5M7p^5292/D>*Q8%kGb`rq0imqKF_Xd@6UhC795D,A$bR`:>\\p@s.;1;_Yu^e-+DR2q:X<tai:]T(6-Z<cW\'$pU[fKq.t*?~>'



#Copyright ReportLab Europe Ltd. 2000-2016
#see license.txt for license details
#history www.reportlab.co.uk/rl-cgi/viewcvs.cgi/rlextra/graphics/guiedit/guidialogs.py
__version__='3.3.0'
'''A specialized combo box based on Pmw's ComboBox class'''
import base64
from reportlab.graphics import renderPM
import Pmw
import collections
try:
	import tkinter
except ImportError:
	import Tkinter as tkinter

class PMDisplay(Pmw.ScrolledFrame):
	def __init__(self, parent,**kw):
		Pmw.ScrolledFrame.__init__(*(self,parent), **kw)
		self.initialiseoptions(PMDisplay)

	def newDrawing(self, zoom, d, text):
		w, h = int(zoom*d.width+0.5), int(zoom*d.height+0.5)
		if zoom!=1.0:
			from reportlab.graphics.shapes import Drawing
			d = Drawing(w,h,d,transform=(zoom,0,0,zoom,0,0))
		dim = d.width,d.height
		ds = renderPM.drawToString(d,fmt='gif')
		ndim = d.width,d.height
		if ndim!=dim:
			print('size changed dim=%r ndim=%r' % (dim,ndim))
			w, h = int(d.width+0.5), int(d.height+0.5)
			ds = renderPM.drawToString(d,fmt='gif')
		self._image = tkinter.PhotoImage(
				data=base64.encodestring(ds),
				width=w,
				height=h,
				format='gif')
		if hasattr(self,'_label'): self._label.grid_forget()
		frame = self.interior()
		self._label = tkinter.Label(frame, width=w, height=h, image=self._image)
		self._label.grid(row=0, column=0, sticky = 'news')
		self['label_text'] = text
		frame.grid_rowconfigure(0, weight = 1)
		frame.grid_columnconfigure(0, weight = 1)
		self.reposition()

def _flipCmd(m,cmd,label,flip,fg):
	i = m.index(label+'*')
	v = (flip.index(m.entrycget(i,'label')[len(label)+1:])+1) % len(flip)
	m.entryconfigure(i,label='%s %s' % (label,flip[v]),foreground=fg[v])
	if cmd: cmd(v)

def _addFlipButton(menu, label='', value=0, flip=['off','on'], fg=['darkred','darkgreen'], command=None):
	menu.add_command(label='%s %s' % (label,flip[value]),foreground=fg[value],
			command=lambda s=_flipCmd,m=menu,label=label,fg=fg,flip=flip,c=command: s(m,c,label,flip,fg))

class RL_MenuBar(Pmw.MainMenuBar):
	def __init__(self,parent,**kw):
		Pmw.MainMenuBar.__init__(*(self,parent), **kw)
		self.initialiseoptions(RL_MenuBar)

	def addFlipButton(self, menuName, statusHelp='', traverseSpec=None, label='', value=0,
		flip=['off','on'], fg=['darkred','darkgreen'],command=None,**kw):
		L = kw['label'] = '%s %s' % (label,flip[value])
		kw['foreground'] = fg[value]
		self.addmenuitem(menuName,'command',statusHelp,traverseSpec,**kw)
		#m = self.component(menuName + '-menu')
		m = self.component(menuName)
		m.entryconfigure(L,command=lambda s=_flipCmd,m=m,label=label,fg=fg,flip=flip,c=command: s(m,c,label,flip,fg))

	def insertmenuitem(self, menuName, index, itemType, statusHelp = '',
			traverseSpec = None, **kw):
		if itemType not in ('command', 'separator', 'checkbutton', 'radiobutton', 'cascade'):
			raise ValueError('unknown menuitem type "%s"' % itemType)
		menu = self.component(menuName)
		if itemType != 'separator':
			self._addHotkeyToOptions(menuName, kw, traverseSpec)
		menu.insert(index,itemType,kw)
		self._menuInfo[menuName][1].insert(index,statusHelp)

class RL_ScrolledText(Pmw.ScrolledText):
	def __init__(self,*args,**kw):
		for a in ('colors','maxlines','autoclear','initText'):
			if a in list(kw.keys()):
				v = kw[a]
				del kw[a]
			else:
				v = None
			setattr(self,a,v)

		Pmw.ScrolledText.__init__(*(self,)+args, **kw)
		w = self._text = self.component('text')
		w.config(state='disabled')
		w.bind('<Button-1>', self._textB1)
		if self.colors:
			self._tag_config(self.colors)
			self._tagNames = tuple(self.colors.keys())
		else:
			self._tagNames = ()
		self.initialiseoptions(RL_ScrolledText)
		if self.initText:
			for k,m in self.initText:
				self.write(msg=m,kind=k)
			del self.initText[:]

	def _textB1(self,event):
		'''special binding for our disabled texts
		Allows dragging to select and copy, but not editing
		'''
		w, x, y = event.widget, event.x, event.y
		call = w.tk.call
		pfx = tkinter.TkVersion>=8.4 and '::tk::' or '::tk'
		call('set',pfx+'Priv(selectMode)','char')
		call('set',pfx+'Priv(mouseMoved)',0)
		call('set',pfx+'Priv(pressX)',x)
		call('set',pfx+'Priv(pressX)',x)
		w.mark_set('insert',call(pfx+'TextClosestGap',str(w),x,y))
		w.mark_set('anchor','insert')
		w.focus()
		w.tag_remove('sel','0.0','end')
		return 'break'


	def write(self,msg, kind=None, flush=0, norm=lambda m: m+(m and(m[-1]!='\n'and'\n'or'')or'\n')):
		w = self._text
		w.config(state='normal')
		if self.autoclear: w.delete('0.0','end')
		w.insert('end',norm(msg),kind)
		if self.maxlines: w.delete('1.0','end-%dl' % self.maxlines)
		w.see('end-2c')
		w.config(state='disabled')
		if flush: w.update()

	def destroy(self):
		if self.initText is not None:
			L = self.initText
			i=0
			for t in self.get('0.0', 'end').split('\n'):
				i = i+1
				for n in self.tag_names('%d.0' % i):
					if n in self._tagNames:
						L.append((n,t))
						break
		Pmw.ScrolledText.destroy(self)

	def clear(self,kind=None):
		w = self._text
		w.config(state='normal')
		if kind is not None:
			rg = w.tag_ranges(kind)
			for i in range(0,len(rg),2):
				w.delete(rg[i],rg[i+1])
		else:
			w.delete('0.0','end')
		w.config(state='disabled')

	def _tag_config(self,V):
		w = self._text
		for k, v in list(V.items()):
			rg = w.tag_ranges(k)
			w.tag_configure(k,v)
			if len(rg): w.tag_add(k,rg[0],rg[1:])
			w.tag_configure(k,v)

class NameDialog(Pmw.Dialog):
	'''
	Dialog window displaying a list and entry field and requesting
	the user to make a selection or enter a value
	'''
	def __init__(self, parent=None, **kw):
		# Define the megawidget options.
		INITOPT = Pmw.INITOPT
		optiondefs = (
			('borderx',    10,	  INITOPT),
			('bordery',    10,	  INITOPT),
			('namelabel',	None,	None),
			)
		self.defineoptions(kw, optiondefs)

		# Initialise the base class (after defining the options).
		Pmw.Dialog.__init__(self, parent)

		# Create the components.
		interior = self.interior()

		aliases = (
			('listbox', 'combobox_listbox'),
			('scrolledlist', 'combobox_scrolledlist'),
			('entry', 'combobox_entry'),
			('label', 'combobox_label'),
			)
		if self['namelabel']:
			self._nameField = self.createcomponent('name',
				(),None,Pmw.EntryField, (interior,),
				labelpos='w',
				label_text = self['namelabel'],
				)
			self._nameField.pack(side='top',expand='true',fill='x',
				padx = self['borderx'], pady = self['bordery'])

		self._combobox = self.createcomponent('combobox',
				aliases, None,
				Pmw.ComboBox, (interior,),
				scrolledlist_dblclickcommand = self.invoke,
				dropdown = 0,
				)

		self._combobox.pack(side='top', expand='true', fill='both',
				padx = self['borderx'], pady = self['bordery'])

		if 'activatecommand' not in kw:
			# Whenever this dialog is activated, set the focus to the
			# ComboBox's listbox widget.
			listbox = self.component('listbox')
			self.configure(activatecommand = listbox.focus_set)

		# Check keywords and initialise options.
		self.initialiseoptions(NameDialog)

	# Need to explicitly forward this to override the stupid
	# (grid_)size method inherited from Tkinter.Toplevel.Grid.
	def size(self):
		return self._combobox.size()

	# Need to explicitly forward this to override the stupid
	# (grid_)bbox method inherited from Tkinter.Toplevel.Grid.
	def bbox(self, index):
		return self._combobox.bbox(index)

Pmw.forwardmethods(NameDialog, Pmw.ComboBox, '_combobox')

class EditTextDialog(Pmw.TextDialog):
	def __init__(self,initialText='',parent=None,**kwds):
		Pmw.TextDialog.__init__(self,parent=parent,**kwds)
		self.insert('end', initialText)
		self.configure(text_state = 'normal')
Pmw.forwardmethods(EditTextDialog, Pmw.ScrolledText, '_text')

class EditListDialog(Pmw.Dialog):
	'''
	Simple dialog to edit a list
	'''
	def __init__(self, parent = None, **kw):
		_getArrowBitmaps(parent)
		# Define the megawidget options.
		INITOPT = Pmw.INITOPT
		optiondefs = (
			('borderx',	10,    INITOPT),
			('bordery',	10,    INITOPT),
			('balloon',	None,	INITOPT),
			)
		self.defineoptions(kw, optiondefs)

		balloon = self['balloon']

		# Initialise the base class (after defining the options).
		Pmw.Dialog.__init__(self, parent)

		# Create the components.
		interior = self.interior()

		aliases = (
			('listbox', 'combobox_listbox'),
			('scrolledlist', 'combobox_scrolledlist'),
			('entry', 'combobox_entry'),
			('label', 'combobox_label'),
			)

		r = 0	#grid row
		self._combobox = self.createcomponent('combobox',
				aliases, None,
				Pmw.ComboBox, (interior,),
				scrolledlist_selectioncommand = None,
				dropdown = 0,
			)
		self._combobox.grid(row=r,column=0,columnspan=4,sticky='news',padx=self['borderx'],pady=self['bordery'])
		interior.rowconfigure(r, weight=1)
		interior.columnconfigure(1, weight=1)
		interior.columnconfigure(2, weight=1)
		self.__listbox = self.component('listbox')
		self._entry = self.component('entry')

		r = 2	#grid row
		b = self._db = tkinter.Button(interior, image='DNArrow',height=1,command = self._dnCmd)
		b.grid(column=0,row=r,sticky='nsew')
		if balloon: balloon.bind(b,'Move selected item down')

		b = self._ab = tkinter.Button(interior, text='add',height=1,command = self._addCmd)
		b.grid(column=1,row=r,sticky='nsew')
		if balloon: balloon.bind(b,'add entered value to list')

		b = self._kb = tkinter.Button(interior, text='del',height=1,command = self._delCmd)
		b.grid(column=2,row=r,sticky='nsew')
		if balloon: balloon.bind(b,'delete selected item')

		b = self._ub = tkinter.Button(interior, image='UPArrow',height=1,command = self._upCmd)
		b.grid(column=3,row=r,sticky='nsew')
		if balloon: balloon.bind(b,'Move selected item up')

		if 'activatecommand' not in kw:
			# Whenever this dialog is activated, set the focus to the
			# ComboBox's listbox widget.
			self.configure(activatecommand = self.__listbox.focus_set)

		#some additional bindings
		self.__listbox.bind('<Down>', self._dnCmd)
		self.__listbox.bind('<Up>', self._upCmd)

		# Check keywords and initialise options.
		self.initialiseoptions(EditListDialog)

	# Need to explicitly forward this to override the stupid
	# (grid_)size method inherited from Tkinter.Toplevel.Grid.
	def size(self):
		return self._combobox.size()

	# Need to explicitly forward this to override the stupid
	# (grid_)bbox method inherited from Tkinter.Toplevel.Grid.
	def bbox(self, index):
		return self._combobox.bbox(index)

	def __edit0(self,event):
		sels = self.getcurselection()
		if len(sels) == 0:
			return None, None
		else:
			item = sels[0]
			return (int(self.curselection()[0]), item)

	def getList(self):
		return self.__listbox.get(0,'end')

	def _addCmd(self,event=None):
		item = self.get().strip()
		if not item: return
		L = self.getList()
		if item in L: return
		i, _ = self.__edit0(event)
		if i is None: i = 0
		self.__listbox.insert(i,item)
		self.__listbox.select_set(i,i)

	def _delCmd(self,event=None):
		i, _ = self.__edit0(event)
		if i is not None:
			self.__listbox.delete(i)
			self.__listbox.select_clear(0,'end')

	def _upCmd(self,event=None):
		i, item = self.__edit0(event)
		if i:
			self.__listbox.delete(i)
			j = i-1
			self.__listbox.insert(j,item)
			self.__listbox.select_set(j,j)

	def _dnCmd(self,event=None):
		i, item = self.__edit0(event)
		if i is not None:
			self.__listbox.delete(i)
			j = i+1
			self.__listbox.insert(j,item)
			self.__listbox.select_set(j,j)

Pmw.forwardmethods(EditListDialog, Pmw.ComboBox, '_combobox')

class FolderField(Pmw.EntryField):
	'''
	EntryField for a folder name and a browse button
	'''
	def __init__(self, parent=None, **kw):
		# Define the megawidget options.
		optiondefs = ()
		self.defineoptions(kw, optiondefs)

		# Initialise the base class (after defining the options).
		Pmw.EntryField.__init__(self, parent)

		# Create the components.
		interior = self.interior()

		b = self.createcomponent('browse', (), 'Button', tkinter.Button,
						(interior,), text='...', command=self._browse)
		b.grid(row=2,column=3,sticky='ew',padx=5)
		interior.grid_columnconfigure(3, weight=0)

		# Check keywords and initialise options.
		self.initialiseoptions()

	def _browse(self):
		path = tkinter.filedialog.askdirectory(initialdir=self.getvalue() or None)
		if path:
			self.setentry(path)

class GoStartDialog(Pmw.Dialog):
	'''
	Simple dialog to get values for the GO command
	'''
	DRIVERS = ['mysql','odbc','mx.ODBC']
	def __init__(self, parent=None, **kw):
		isCSV = kw.pop('isCSV')
		for k,v in (
				('title','DataAwareDrawing Parameters'),
				('buttons',('OK','Cancel')),
				('defaultbutton','OK'),
				):
			kw.setdefault(k,v)
		# Define the megawidget options.
		INITOPT = Pmw.INITOPT
		optiondefs = (
			('balloon',	None,	INITOPT),
			('icwd',	'',	INITOPT),
			('isql',	'',	INITOPT),
			('ishow',	0, INITOPT),
			('iformats','', INITOPT),
			('imaxcharts',	'', INITOPT),
			('idb',    '',	INITOPT),
			('ipattern','',	INITOPT),
			('ioutdir',	'',INITOPT),
			)
		if not isCSV:
			optiondefs += (
			('ihost',	'', INITOPT),
			('iuser',	'', INITOPT),
			('ipasswd', '', INITOPT),
			('idriver', 0, INITOPT),
			)
		self.defineoptions(kw, optiondefs)

		# Initialise the base class (after defining the options).
		Pmw.Dialog.__init__(self, parent)

		# Create the components.
		interior = self.interior()
		for r,name in enumerate(('cwd','sql')):
			klass = name=='sql' and Pmw.EntryField or FolderField
			value=self['i'+name] or ''
			w = self.createcomponent(name,(),None,klass,interior,
				labelpos='w',
				label_text = name.upper(),
				value=value,
				)
			w.grid(row=r,column=0,columnspan=5,sticky='ew',padx=5,pady=5)
			interior.grid_rowconfigure(r, weight=0)
			setattr(self,'_'+name,w)
		r += 1
		('host','driver','user','passwd','db')
		for c,name in enumerate(isCSV and ('db',) or ('host','driver','user','passwd','db')):
			KW = dict(labelpos='n',label_text=name.capitalize())
			if name=='driver':
				klass = Pmw.OptionMenu
				KW['items']=self.DRIVERS
				KW['initialitem']=self['i'+name]
			else:
				klass = Pmw.EntryField
				KW['entry_width'] = name=='host' and 32 or 12
				KW['value']=self['i'+name] or ''
			if name=='passwd':
				KW['entry_show']='*'
			w = self.createcomponent(name,(),None,klass,interior,**KW)
			w.grid(row=r,column=c,sticky='ew',pady=5,padx=2)
			interior.grid_columnconfigure(c, weight=1)
			setattr(self,'_'+name,w)
		interior.grid_rowconfigure(r, weight=0)

		r += 1
		klass = Pmw.EntryField
		for c,name in enumerate(('formats','outdir','pattern','maxcharts')):
			w = self.createcomponent(name,(),None,klass,interior,
			labelpos='n',
			label_text=name.capitalize(),
			entry_width=name=='maxcharts' and 3 or 12,
			value=self['i'+name]) or ''
			w.grid(row=r,column=c,sticky=name!='maxcharts' and 'ew' or '',pady=5,padx=2)
			interior.grid_columnconfigure(c, weight=1)
			setattr(self,'_'+name,w)

		self._showVar = tkinter.IntVar()
		self._showVar.set(self['ishow'])
		F = tkinter.Frame(interior)
		tkinter.Label(F,text='Show Preview').pack(side='top')
		w = self.createcomponent('show',(),None,tkinter.Checkbutton,F,
				text = '',
				onvalue='1',
				offvalue='0',
				variable=self._showVar,
				)
		w.pack(side='top')
		setattr(self,'_ishow',w)
		F.grid(row=r,column=4,pady=5,padx=2)
		interior.grid_rowconfigure(r, weight=0)
		# Check keywords and initialise options.
		self.initialiseoptions()

dnColor='blue'
upColor='red'

class AttrListBox(Pmw.MegaWidget):
	def __init__(self, parent = None, **kw):
		_getArrowBitmaps(parent)

		# Define the megawidget options.
		INITOPT = Pmw.INITOPT
		optiondefs = (
			('labelmargin',			0,		  INITOPT),
			('labelpos',			None,	INITOPT),
			('listheight',			200,	INITOPT),
			('selectioncommand',	None,	None),
			('dblclickcommand',		   None,	None),
			('upcommand',			None,	None),
			('dncommand',			None,	None),
			('unique',				  1,		INITOPT),
		)
		self.defineoptions(kw, optiondefs)

		# Initialise the base class (after defining the options).
		Pmw.MegaWidget.__init__(self, parent)

		# Create the components.
		interior = self.interior()

		# Create the scrolled listbox
		self._list = self.createcomponent('scrolledlist',
				(('listbox', 'scrolledlist_listbox'),), None,
				Pmw.ScrolledListBox, (interior,),
				selectioncommand = self._selectCmd,
				dblclickcommand = self._dblCmd)
		self._list.grid(column=2, row=2, sticky='nsew')
		self.__listbox = self._list.component('listbox')

		ef = self.createcomponent('entryframe',
				(), None,
				tkinter.Frame, (interior,),
				)

		self._db = tkinter.Button(ef, image='DNArrow',command = self._dnCmd)
		self._db.grid(column=0,row=0,sticky='nsew')
		self._entryfield = self.createcomponent('entryfield',
				(('entry', 'entryfield_entry'),), None,
				Pmw.EntryField, (ef,))
		self._entryfield.grid(column=1, row=0, sticky='nsew')
		self._ub = tkinter.Button(ef, image='UPArrow',command = self._upCmd)
		self._ub.grid(column=2,row=0,sticky='nsew')

		ef.grid_columnconfigure(1, weight = 1)
		self._entryWidget = self._entryfield.component('entry')
		self.__listbox.configure(font=('Courier',10))
		ef.grid(column=2, row=3, sticky='nsew')

		interior.grid_columnconfigure(2, weight = 1)

		# The scrolled listbox should expand vertically.
		interior.grid_rowconfigure(2, weight = 1)

		# Create the label.
		self.createlabel(interior, childRows=2)

		self._entryWidget.bind('<Down>', self._next)
		self._entryWidget.bind('<Up>', self._previous)
		self._entryWidget.bind('<Control-n>', self._next)
		self._entryWidget.bind('<Control-p>', self._previous)
		self.__listbox.bind('<Down>', self._next)
		self.__listbox.bind('<Up>', self._previous)
		self.__listbox.bind('<Control-n>', self._next)
		self.__listbox.bind('<Control-p>', self._previous)

		# Check keywords and initialise options.
		self.initialiseoptions(AttrListBox)

	def destroy(self):
		Pmw.MegaWidget.destroy(self)

	#======================================================================
	# Public methods
	def get(self, first = None, last=None):
		if first is None:
			return self._entryWidget.get()
		else:
			return self._list.get(first, last)

	def invoke(self):
		return self._selectCmd()

	def _logAttrInfo(self):
		from rlextra.graphics.guiedit.guiedit import editor, _findLHS, _stripSub, exceptionLog
		attrHelp = getattr(editor,'attrHelp',None)
		if not attrHelp: return
		try:
			from reportlab.graphics.widgetbase import TypedPropertyCollection
			name = editor.getFullObjectName()
			aname = self._lastsetentry[1]
			unsub = aname[0]!='['
			lhs = _findLHS(name+(unsub and '.' or '')+aname)
			if lhs!=None: lhs = lhs[5:]
			if not unsub:
				sname, aname = _stripSub(aname)
				sname = '[%s].%s' % (sname,aname)
			else:
				sname = aname = _findLHS(aname)
			obj = editor.getSampleAttr(editor.attrRootName)
			if isinstance(obj,TypedPropertyCollection):
				obj = getattr(obj,'_value',obj)
			am = getattr(obj,'_attrMap',None)
			inMap = am and aname in am or 0
			value = editor.getSampleAttr(lhs)
			if inMap:
				from reportlab.lib.validators import isBoolean, OneOf
				ame = am[aname]
				desc = getattr(ame,'desc')
				I = []
				if desc:
					for l in desc.split('\n'):
						I.append(('    ')+l)
				validate = ame.validate
				if validate is isBoolean:
					if I: I.append('    boolean value=%s' % value)
				elif isinstance(validate,OneOf):
					if I: I.append('    one of %r\n    current value=%s' % (validate._enum,value))
				else:
					if I: I.append('    current value=%s' % repr(value))
				if I:
					I = '\n'.join([aname]+I)
					attrHelp.write(I,flush=1,kind='temporary')
					return
			attrHelp.write('\n%s\n    %s' % (aname,value in getattr(obj,'contents',[]) and 'child widget' or 'no attribute information'),kind='temporary',flush=1)
		except:
			exceptionLog('Error trying to show attibute info')

	def selectitem(self, index, setentry=1):
		if type(index) == bytes:
			text = index
			items = self._list.get(0, 'end')
			if text in items:
				index = list(items).index(text)
			else:
				raise IndexError('index "%s" not found' % text)
		elif setentry:
			text = self._list.get(0, 'end')[index]

		self._list.select_clear(0, 'end')
		self._list.select_set(index, index)
		self._list.activate(index)
		self.see(index)
		if setentry:
			self._lastsetentry = (index, text)
			self._entryfield.setentry(text)
			self._logAttrInfo()

	# Need to explicitly forward this to override the stupid
	# (grid_)size method inherited from Tkinter.Frame.Grid.
	def size(self):
		return self._list.size()

	# Need to explicitly forward this to override the stupid
	# (grid_)bbox method inherited from Tkinter.Frame.Grid.
	def bbox(self, index):
		return self._list.bbox(index)

	def clear(self):
		self._entryfield.clear()
		self._list.clear()

	# Private methods
	def _next(self, event):
		size = self.size()
		if size <= 1:
			return

		cursels = self.curselection()

		if len(cursels) == 0:
			index = 0
		else:
			index = int(cursels[0])
			if index == size - 1:
				index = 0
			else:
				index += 1

		self.selectitem(index)
		if event.widget is self._list._listbox: return 'break'

	def _previous(self, event):
		size = self.size()
		if size <= 1:
			return

		cursels = self.curselection()

		if len(cursels) == 0:
			index = size - 1
		else:
			index = int(cursels[0])
			if index == 0:
				index = size - 1
			else:
				index -= 1

		self.selectitem(index)
		if event.widget is self._list._listbox: return 'break'

	def __doCmd(self,event,cmdName):
		sels = self.getcurselection()
		if len(sels) == 0:
			item = None
		else:
			item = sels[0]
			self._lastsetentry = (int(self.curselection()[0]), item)
			self._entryfield.setentry(item)
			self._logAttrInfo()
		cmd = self[cmdName]
		if isinstance(cmd, collections.Callable):
			return cmd(item)

	def _selectCmd(self, event=None):
		return self.__doCmd(event,'selectioncommand')

	def _dblCmd(self, event=None):
		return self.__doCmd(event,'dblclickcommand')

	def _upCmd(self,event=None):
		return self.__doCmd(event,'upcommand')

	def _dnCmd(self,event=None):
		return self.__doCmd(event,'dncommand')

Pmw.forwardmethods(AttrListBox, Pmw.ScrolledListBox, '_list')
Pmw.forwardmethods(AttrListBox, Pmw.EntryField, '_entryfield')

def _getArrowBitmaps(master):
	if not master: master = tkinter._default_root
	master.tk.call('image','create', 'bitmap', 'DNArrow', '-foreground', dnColor,
'-data',
'''#define dnarrow_width 8
#define dnarrow_height 8
#define dnarrow_x_hot 0
#define dnarrow_y_hot 0
static char dnarrow_bits[] = {
0x0, 0x3c, 0x3c, 0xff, 0x7e, 0x3c, 0x18, 0x0,
};''')
	master.tk.call('image','create', 'bitmap', 'UPArrow', '-foreground', upColor,
'-data',
'''#define uparrow_width 8
#define uparrow_height 8
#define uparrow_x_hot 0
#define uparrow_y_hot 0
static char uparrow_bits[] = {
0x0, 0x18, 0x3c, 0x7e, 0xff, 0x3c, 0x3c, 0x0,
};''')



__all__=('NamedTuple',)
from operator import itemgetter
import sys

def NamedTuple(typename, field_names):
    """Returns a new subclass of tuple with named fields.

    >>> Point = PNamedTuple('Point', 'x y')
    >>> Point.__doc__           # docstring for the new class
    'Point(x, y)'
    >>> p = Point(11, y=22)     # instantiate with positional args or keywords
    >>> p[0] + p[1]             # works just like the tuple (11, 22)
    33
    >>> x, y = p                # unpacks just like a tuple
    >>> x, y
    (11, 22)
    >>> p.x + p.y               # fields also accessable by name
    33
    >>> p                       # readable __repr__ with name=value style 
    Point(x=11, y=22)
    >>> p.__field_names__
    ('x', 'y')
    """

    if isinstance(field_names,str): field_names = field_names.split()
    nargs = len(field_names)

    def __new__(cls, *args, **kwds):
        if kwds:
            try:
                args += tuple(kwds[name] for name in field_names[len(args):])
            except KeyError as name:
                raise TypeError('%s missing required argument: %s' % (typename, name))
        if len(args) != nargs:
            raise TypeError('%s takes exactly %d arguments (%d given)' % (typename, nargs, len(args)))
        return tuple.__new__(cls, args)

    def __from_iterable__(cls,arg):
        return cls.__new__(cls,*arg)

    repr_template = '%s(%s)' % (typename, ', '.join('%s=%%r' % name for name in field_names))

    m = dict(vars(tuple))       # pre-lookup superclass methods (for faster lookup)
    m.update(__doc__= '%s(%s)' % (typename, ', '.join(field_names)),
             __slots__ = (),    # no per-instance dict (so instances are same size as tuples)
             __new__ = __new__,
             __repr__ = lambda self, _format=repr_template.__mod__: _format(self),
             __module__ = sys._getframe(1).f_globals['__name__'],
             __field_names__ = tuple(field_names),
             __from_iterable__=classmethod(__from_iterable__),
             )
    m.update((name, property(itemgetter(index))) for index, name in enumerate(field_names))

    return type(typename, (tuple,), m)

def _test():
    import doctest
    return doctest.testmod()

if __name__ == "__main__":
    _test()



import string, traceback, re, os
_rcrc_re=re.compile(r'=(?P<sh>\w+)!R(?P<r1>\d+)C(?P<c1>\d+)(?::R(?P<r2>\d+)C(?P<c2>\d+)|)',re.I)

def AddWorkBook(xl):
	if string.atoi(xl.Version[0])>=8:
		wb=xl.Workbooks.Add()
	else:
		wb=xl.Workbooks().Add()
	return wb

def DeleteSheet(wb,name):
	a=wb.Application
	da=a.DisplayAlerts
	if da: a.DisplayAlerts=0
	try:
		wb.Sheets(name).Delete()
	except:
		pass
	if da: a.DisplayAlerts=da

def NewSheet(wb,name):
	DeleteSheet(wb,name)
	sh=wb.Sheets.Add()
	sh.Name=name
	return sh

_ALPHABET="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
def	Letter(i):
	i = i - 1
	j = i % 26
	i = i / 26
	if i>0:
		return _ALPHABET[i-1]+_ALPHABET[j]
	else:
		return _ALPHABET[j]

def RC(i,j):
	return Letter(j)+str(i)

def GetRange(sh, i,j,k,l):
	return sh.Range(RC(i,j)+':'+RC(k,l))

def WriteMX(sh,r,c,MX, label=None, fmt=None):
	if not label is None:
		sh.Cells(r,c).Value=label
		r=r+1
		c=c+1
	n,m = MX.shape
	rfmt="%s%%d:%s%%d"%(Letter(c),Letter(c+m-1))
	for i in range(n):
		sh.Range(rfmt%(r+i,r+i)).Value=tuple(MX[i,:].tolist())
	if not fmt is None: sh.Range(rfmt%(r,r+n)).NumberFormat=fmt
	return (r+n,c+m)

def _NoneToDefault(r,NoneValue=0):
	if type(r)==type(()): r=list(r)
	while 1:
		try:
			i=r.index(None)
		except ValueError:
			return r
		n=r[i:].count(None)
		r[i:i+n]=[NoneValue]*n

def _ReadMX(sh,r,c, n,m, NoneValue=0, typecode='d'):
	from Numeric import array, zeros
	A,B = Letter(c),Letter(c+m-1)
	try:
		cell1="%s%d" % (A,r)
		cell2="%s%d" % (B,r+n-1)
		def filt(x,d=0):
 			if x is None: return d
 			return x
		MX=array(list(map(lambda x,filt=filt: list(map(filt,x)),sh.Range(cell1,cell2).Value)),typecode)
	except:
		MX = zeros((n,m),typecode)
		fmt="%s%%d:%s%%d"%(A,B)
		for i in range(n):
			R=sh.Range(fmt%(r+i,r+i)).Value
			if m>1:
				R = R[0]
			try:
				MX[i,:]=array(R,typecode)
			except:
				MX[i,:]=array(_NoneToDefault(R,NoneValue),typecode)
	return MX,(r+n,c+m)

def ReadMX(sh,r,c, n,m, NoneValue=0, typecode='d'):
	return _ReadMX(sh,r,c, n,m, NoneValue=NoneValue, typecode=typecode)[0]

def splitNR(wb,nr):
	'''splits a named range from wb'''
	m = _rcrc_re.match(str(wb.Names(nr).RefersToR1C1))

	sh,r,c,n,m = m.group('sh'),int(m.group('r1')),int(m.group('c1')),m.group('r2'),m.group('c2')
	if n is not None:
		n = int(n) - r + 1
		m = int(m) - c + 1
	else:
		n = m = 1
	return sh, r,c, n,m

def readNR(wb,nr):
	'''returns a value from wb correponding to a named range'''
	sh, r,c, n,m = splitNR(wb,nr)
	return wb.Sheets(sh).Cells(r,c).Value

def readNRMX(wb,nr,NoneValue=0,typecode='d'):
	'''returns a matrix from wb correponding to a named range'''
	sh, r,c, n,m = splitNR(wb,nr)
	return _ReadMX(wb.Sheets(sh),r,c, n,m, NoneValue=NoneValue, typecode=typecode)[0]

def writeNRMX(wb,nr,mx):
	sh, r,c, n,m = splitNR(wb,nr)
	A,B = Letter(c),Letter(c+m-1)
	cell1="%s%d" % (A,r)
	cell2="%s%d" % (B,r+n-1)
	wb.Sheets(sh).Range(cell1,cell2).Value = mx[0:n,0:m]


def NumValid(x):
	if x is None: return 0
	try:
		if string.strip(str(x))=='': return 0
		return 1
	except:
		return 0

def CheckMinMax(sh, lim, refs):
	if lim is None: return
	v=(sh.Cells(refs[0][0],refs[0][1]).Value, sh.Cells(refs[1][0],refs[1][1]).Value)
	if NumValid(v[0]) and NumValid(v[1]):
		try:
			if v[0]<=v[1] and v[0]>=lim[0] and v[1]<=lim[1]: return
		except:
			pass
	raise ValueError("Bad dimensions in sheet %s\n\rlim=%s refs=%s v=%s"%(sh.Name,str(lim),str(refs),str(v)))

def	CCount(sh,row,start,lim=None):
	i=start
	while NumValid(sh.Cells(row,i).Value):
		i=i+1
	CheckMinMax( sh, lim, ((row, start), (row, i-1)))
	return i-start,i+1

def	RCount(sh,col,start,lim=None):
	i=start
	while NumValid(sh.Cells(i,col).Value):
		i=i+1
	CheckMinMax( sh, lim, ((start,col), (i-1,col)))
	return i-start,i+1

def TextBoxChange(TB,msg,append=0):
	try:
		locked = TB.Locked
		if locked: TB.Locked=0
		if append:
			TB.Value = TB.Value + msg
		else:
			TB.Value = msg

		if locked: TB.Locked=locked
	except:
		traceback.print_exc()

def getExcel():
	from win32com.client import Dispatch
	return Dispatch('Excel.Application')

def ensureOpen(xl,fn):
	fn = str(fn)
	if not os.path.isabs(fn):
		fn = os.path.abspath(fn)
	fn = string.lower(os.path.normcase(os.path.normpath(fn)))
	WB = xl.WorkBooks
	n = WB.Count
	while n:
		wb = WB(n)
		n -= 1
		if string.lower(str(wb.fullName))==fn: return wb

	return WB.Open(fn)

def normalizeLabel(w):
	return ''.join([_f for _f in w.lower().split() if _f])

def normalizeAndFixLabels(L, R={}):
	f = lambda x, K=list(R.keys()), R=R: x in K and R[x] or x
	return list(map(lambda n,f=f:''.join(map(f, [_f for _f in n.lower().split() if _f])),L))

class IndexedRow:
	'''a set of values with named indeces'''
	def __init__(self,indeces,values,norm=normalizeLabel):
		assert len(indeces)==len(values)
		self.__dict__['_indeces'], self.__dict__['_values'] = list(map(norm,list(indeces))), list(values)
		self.__dict__['_norm'] = norm

	def __getitem__(self,x):
		return self._values[self._indeces.index(self._norm(x))]

	def __setitem__(self,x,v):
		self._values[self._indeces.index(self._norm(x))] = v

	__getattr__ = __getitem__
	__setattr__ = __setitem__

	def index(self,x):
		return self._indeces.index(self._norm(x))



__code__='Gat=(s)6@U\'RuIZTe-[Fl.FWrGVHRBd[WW_1elFU(=EZ6bplTs$C`Tun(O3k[Q36QWDn<rdXKkV[D-PHSSaOA1o.SM48\\I&kKC4#754=WPHJ9V.fQ8p[p]F/)[\\i\\f>-pP!@%VC^^9MTjDU+h5l*>=*Pa\\N(>q3gH,\'U!7DSMF(B?Z"]Ei`gE!c"hi%j4*^_OWq!NuQR"Sa_028Duf56\\\\0GQ[Fk^mi8_ks^fH*r^:&rZl@d7WT\\VX?\\fhkMW-BioNBYI."dk@Wqi;B;m-(E!lm;0?Ak[\'S_aK/FqGhlW:N?S&-mF:KfhjE)_,qD!JE96@\'WYSLI=?_VqIS]"3j\\F1DDn$b6&5$/0tP2]78O(;GK>e2\'fL=Z^TXJG_^H<H)MF"GkQn\\KpO2o+Zg=7BlU_;1GY@E.+e3N92+6?)^2nQ/RSm0I`fg/ZMrKhblqtj]od(/hK&+:U4KDUkZRAMd=f)EBA$D/K5*gQ,ne_iO<UZR^_!"<7ImZ3_%.]d4sBYJmNn:Zh\']lMKrFb7BnD0KI%R`H/:2So3)^$iE-;8=gS_"$r+U"([U0T\'.28@oq/(d!cPF%.OiI`nf%^EQ"s_O_\\O4_-S\'[#(mkaNh8_:sPkN"3GLM$sE9"eEh\'A"k>"EaV7=lcOAihgGp9`+pluW^J%m=JOPAo97ZE`SYaF2rj2"hEr`.U1s;E%OWSE-q!`$ifi:;\\e<V)i1fDnU9LV+e%-+n*\\@)L!OoIChEH++q%]+%TD9<h3g)P!lA.,]*8<enIJd_`J!ts%RR\\W_%>([@;&K^JhGAM9aA*MY=nRH=cqBhcuuIhWiosqVAp`rHJ"fI]/R(R,q8h+/N/@P[mpj]k:XB-/HLL?Cqk:35;J(=Lcmu$/,!r&rk;1N:u:R\\#stdkk?W8r>+[BGc)tcr9h.!R9@!AmS4TH&hAJslo/0faS)ZQn#=8HaN1;^+1Y2-/?snglFtL"Co_G3r&%3_in!WdfZs7!WWo6ahS\\0F6(N!-L:3XlIXe5Q<JM1H8CM!odj/Ne96oSX9[7t4=E.`"Dm4-"3hZX7SaT?WL@+fJ[fo\'b,c%K*b%WEVb)E_J(Ns6E`e.3,D[JUlHj8CXjJT:BrQHP+2dCfOV6qE]7-f@%Y;K?KX77:q<2%D~>'



# rml_shellext_win32
# adapted from Adam Twardoch's script in fonttools which
# does exactly the analogous thing!  Thanks,  Adam

# This script installs a Windows 9x/NT shell extension for RML files.
# After installing it, click with the right mouse button on an RML file
# and choose "Convert RML to PDF" from the context menu to run rml2pdf
# over it. 
# or "Convert all RML to PDF" to convert all RML files in the current directory. 


import sys, os, string, tempfile
def run():
  if not sys.platform == 'win32':
    print('This program is for Win32 (Windows 9x/NT) systems only.')
    sys.exit(2)

  # get the folder where Python resides.  assume rml2pdf.exe is here
  pythondir = sys.exec_prefix

  # escape backslashes (regedit.exe requires that the reg files are formatted that way)
  pythondir = string.replace(pythondir, '\\', '\\\\')

  # Prepare the text to write to the temporary reg file
  regtext = r"""REGEDIT4


  [HKEY_CLASSES_ROOT\.rml]
  @="rmlfile"

  [HKEY_CLASSES_ROOT\rmlfile]
  @="RML Document"

  [HKEY_CLASSES_ROOT\rmlfile\shell]
  @=""

  [HKEY_CLASSES_ROOT\rmlfile\shell\Convert current RML to PDF]
  @="Convert current RML to PDF"

  [HKEY_CLASSES_ROOT\rmlfile\shell\Convert current RML to PDF\command]
  @="\"%(pythondir)s\\rml2pdf.exe\"  \"%%1\""

  """ % locals()


  # Create the temporary reg file which will be joined into the Windows registry
  reg_file_name = os.path.join(os.path.dirname(tempfile.mktemp()), "~rmltemp.reg")
  reg_file = open(reg_file_name, "w")
  reg_file.write(regtext)
  reg_file.close()

  # Join the temporary reg file into the Windows registry
  execline = '%windir%\\regedit.exe ' + reg_file_name

  file = os.popen(execline)
  output = ""
  while 1:
    chunk = file.read(1000)
    if not chunk:
      break
      output = output + chunk

  print(output)

  os.remove(reg_file_name)

if __name__=='__main__':
  run()


# benchmark



# MSXML:  This can be downloaded from many places.  You need 3.0
# which is NOT in most newly installed Windows boxes. (650kb)
# http://download.microsoft.com/download/xml/Install/3.0/WIN98Me/EN-US/msxml3.exe
#    for a quick tutorial on MSXML 3.0, see
# http://www.perfectxml.com/articles/xml/msxml30.asp

# you should then run the COM MakePY utility on the Pythonwin menu.
# to get it going as fast as possible.


## apparently time.clock is the fine grained one e.g.
##>>> time.clock() - time.clock()
##-1.1733331545471515e-005
##>>> time.time() - time.time()
##0.0
##>>> 
import sys
import glob
import time
from types import TupleType
from reportlab.lib.utils import getStringIO
    
def tupleTreeStats(node):
    # counts tags and attributes recursively
    # use for all reportlab parsers
    if node[1] is None:
        attrCount = 0
    else:
        attrCount = len(node[1])
    nodeCount = 1
    if node[2] is not None:
        for child in node[2]:
            if type(child) is TupleType:
                a, n = tupleTreeStats(child)
                attrCount = attrCount + a
                nodeCount = nodeCount + n
    return attrCount, nodeCount

###  pyRXP - our wrapper around Univ of Edinburgh

def getPyRXPParser():
    import pyRXP
    p = pyRXP.Parser()
    return p


def parseWithPyRXP(parser, rawdata):
    return parser.parse(rawdata)

###  rparsexml - Aaron's very fast pure python parser

def loadRparseXML():
    #it's a module, what the heck
    from reportlab.lib import rparsexml
    return rparsexml

def parseWithRParseXML(rparsexml, rawdata):
    #first argument is a dummy holding none
    return rparsexml.parsexml0(rawdata)[0] 

###  expattree - Andy's expat parser
def loadExpatTree():
    from rlextra.radxml.expattree import ExpatTreeParser
    return ExpatTreeParser

def parseWithExpatTree(ExpatTreeParser,text):
    P = ExpatTreeParser()
    return P.parseText(text)

####### minidom - non-validating DOM parser in the Python distro

def loadMiniDOM():
    import xml.dom.minidom
    return xml.dom.minidom

def parseWithMiniDOM(dom_module, rawdata):
    #parser is None
    return dom_module.parseString(rawdata)
    
def statsWithMiniDOM(node):
    return (1, 0)

#########  Microsoft XML Parser via COM ######################


def loadMSXML30():
    from win32com.client import Dispatch
    msx = Dispatch('Microsoft.XMLDOM')
    return msx

def parseWithMSXML30(msx, rawdata):
    msx.loadXML(rawdata)
    return msx

def statsWithMSXML30(node):
    #not done
    return (1,0)    

###########4DOM ###############
def load4DOM():
    from xml.dom.ext.reader import PyExpat
    from xml.dom import Node
    reader = PyExpat.Reader()
    return reader

def parseWith4DOM(reader, rawdata):
    return reader.fromString(rawdata)


def statsWith4DOM(node):
    #node
    return (1,0)

def loadCDomlette():
    from Ft.Lib import cDomlettec
    return cDomlettec

def parseWithCDomlette(modul, rawdata):
    io = getStringIO(rawdata)
    return modul.parse(io, '')

def statsWithCDomlette(node):
    #node
    return (1,0)

##########put them all together################

TESTMAP = [
    # name of parser; function to initialize if needed;
    # function to parse; function to do stats
    ('pyRXP', getPyRXPParser, parseWithPyRXP, tupleTreeStats),
    ('rparsexml', loadRparseXML, parseWithRParseXML, tupleTreeStats),
    ('expattree', loadExpatTree, parseWithExpatTree, tupleTreeStats),
    ('minidom', loadMiniDOM, parseWithMiniDOM, statsWithMiniDOM),
    ('msxml30', loadMSXML30, parseWithMSXML30, statsWithMSXML30),
    ('4dom', load4DOM, parseWith4DOM, statsWith4DOM),
    ('cdomlette', loadCDomlette, parseWithCDomlette, statsWithCDomlette)
    ]    

def runtest(sampleText,num,memPause):
    (name, loadFunc, parseFunc, statFunc) = TESTMAP[num-1]
    print('testing %s' % name)
    #load the parser
    t0 = time.clock()
    try:
        parser = loadFunc()
    except:
        print('!!!!!Load Failed')
        return
    loadTime = time.clock() - t0
    
    if memPause: baseMem = float(input("Please input process base memory in kb >"))
    t1 = time.clock()
    parsedOutput = parseFunc(parser, sampleText)
    t2 = time.clock()
    parseTime = t2 - t1
    
    if memPause:
        totalMem = float(input('Please enter memory allocation in kb >'))
        usedMem = totalMem - baseMem
        memFactor = usedMem * 1024.0 / len(sampleText)
    t3 = time.clock()
    n, a = statFunc(parsedOutput)
    t4 = time.clock()
    traverseTime = t4 - t3
    print('%d tags, %d attributes' % (n, a))
    if memPause:
        print('%s: init %0.4f, parse %0.4f, traverse %0.4f, mem used %dkb, mem factor %0.2f' % (
            name, loadTime, parseTime, traverseTime, usedMem, memFactor))
    else:
        print('%s: init %0.4f, parse %0.4f, traverse %0.4f' % (
            name, loadTime, parseTime, traverseTime))
    print()

def interact():
    print("""
    Interactive benchmark suite for Python XML parsers.
    Parsers available:
    """)
    auto = '--auto' in sys.argv
    if auto: sys.argv.remove('--auto')
    try:
        sample = sys.argv[1]
    except:
        sample = 'rml_a.xml'

    sampleText = open(sample).read()
    i = 1
    for (name, a, b, c) in TESTMAP:
        print('\t%d.  %s' % (i, name))
        i = i + 1
    if not auto:
        print()
        inp = input("Shall we do memory tests?  i.e. I pause while you look at Task Manager? y/n  ")
        assert inp in 'yn', 'enter "y" or "n".  Please run again!'
        pause = (inp == 'y')
        inp = input('Test number (or x to exit)>')
        if inp == 'x':
            print('bye')
            return
        else:
            runtest(sampleText,int(inp),pause)
    else:
        for num in range(1,len(TESTMAP)+1):
            runtest(sampleText,num,0)

if __name__=='__main__':
    interact()



#Autogenerated by ReportLab guiedit do not edit
import os, sys, __main__
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin, String
from reportlab.rl_config import T1SearchPath, TTFSearchPath
import reportlab, rlextra

class DiagnosticDrawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(*(self,width,height)+args, **kw)
		fs = 10
		ld = 1.2*fs
		sysL = ['byteorder', 'copyright', 'displayhook''exec_prefix', 'executable', 'hexversion', 'maxint', 'maxunicode', 'platform', 'prefix', 'version', 'version_info', 'warnoptions']
		n0 = 7+len(sysL)
		self.width=6.5*72
		self.height = (n0+len(sys.path)+len(sys.argv)+len(T1SearchPath)+len(TTFSearchPath))*ld
		self._add(self,String(10,self.height-ld,'cwd="%s"' % os.getcwd(),fontSize=fs), name=None,validate=None,desc=None)
		self._add(self,String(10,self.height-2*ld,'argv[0]="%s"' % sys.argv[0],fontSize=fs), name=None,validate=None,desc=None)
		self._add(self,String(10,self.height-3*ld,'sample path="%s"' % getattr(__main__,'_guiedit_sample_path','<unknown>'),fontSize=fs), name=None,validate=None,desc=None)
		self._add(self,String(10,self.height-4*ld,'_rl_embed_location="%s"' % getattr(__main__,'_rl_embed_location','<unknown>'),fontSize=fs), name=None,validate=None,desc=None)
		self._add(self,String(10,self.height-5*ld,'reportlab location="%s"' % os.path.dirname(reportlab.__file__),fontSize=fs), name=None,validate=None,desc=None)
		self._add(self,String(10,self.height-6*ld,'rlextra location="%s"' % os.path.dirname(rlextra.__file__),fontSize=fs), name=None,validate=None,desc=None)
		if 'do error' in sys.argv: raise ValueError('Deliberate Error For Testing')
		for i in range(len(sysL)): self._add(self,String(10,self.height-(7+i)*ld,'sys.%s="%s"' % (sysL[i],getattr(sys,sysL[i],'<missing>')),fontSize=fs), name=None,validate=None,desc=None)
		for i in range(len(sys.argv)): self._add(self,String(10,self.height-(n0+i)*ld,'sys.argv[%d]="%s"' % (i,sys.argv[i]),fontSize=fs), name=None,validate=None,desc=None)
		for i in range(len(sys.path)): self._add(self,String(10,self.height-(n0+i+len(sys.argv))*ld,'sys.path[%d]="%s"' % (i,sys.path[i]),fontSize=fs), name=None,validate=None,desc=None)
		for i in range(len(T1SearchPath)): self._add(self,String(10,self.height-ld*(n0+i+len(sys.argv)+len(sys.path)),'T1SearchPath[%d]="%s"' % (i,T1SearchPath[i]),fontSize=fs), name=None,validate=None,desc=None)
		for i in range(len(TTFSearchPath)): self._add(self,String(10,self.height-ld*(n0+i+len(sys.argv)+len(sys.path)+len(T1SearchPath)),'TTFSearchPath[%d]="%s"' % (i,TTFSearchPath[i]),fontSize=fs), name=None,validate=None,desc=None)

if __name__=="__main__": #NORUNTESTS
	DiagnosticDrawing().save(formats=['pdf'],outDir='.',fnRoot=None)



__code__='Gas1];/[CB&>$Qs4,t7`j+]moV+40%7cKpr:a(K\'Q0VJ5/gdUQ#.^DUj^s]]F2#9SXV9l^5,AlFr"8o45N?)G(HaEnnMR14Z$/_(h3m`$)=P%a_qaDZc(AZ]J!%QO0EQKP!YF9IYWX^K,X;\\r!FAHL#AF1d0EEc@)IOS)!O)U7#hT9m!4W1,*oKK&97SV0pAs$!QN<ER56/=tp`\\QcJ6)b9#5eVn)W5bR:B>AFbG?"%F$9<c)Z34_(5uuNl^WO55h=l3"S)$nfQqGOF(2TZ5N>^5Pf$34G!7Rt<\\RmR=V]#Cb-D:&CIAnn=WO[LN(j42lrZlAf,ub[_:EGr(grmr,iDe?Puf;GJ7BKEK2><G+1.tibVDh.^p*q)rQ8,Z1J\\,Zi[:`lI_iXeH(0lH#>b1,*^8hU`PSe-^=K;daC^?6Wkqk^XMW+jZr;dj0ZVl7aOlOQ"P\'/ur`N*\\S8./8^#lCVX5-GQh:YPIU;JDF::?>lSb/j7dZj(?3"/2RZpdXKes;MsK\\u:3fH3c4/"3%g:2gj9HHGbb:Aj>J6o?Er,b6`LCq+0`6aKo.,P1nRQuCW<AXY%7qV1_@qfHM,RI/a+(s"\\#BW$_Y!c9;HF+/I>Vr)^1LPLYF.Q8T%gBj%J@h\\"?c(4ib+&.,\'-F\'*i>>EXD&QY!h8,ia%641"HaN[e4M,KO:=6/QApN!n:5M\'fu^\\(WY#?#-\'R8+B!8YXY2XrD8-F"Zu#l8tY<iF[PJQc.UU^764I][MlN;:t,$"ab\\F*=CR=N%Wsm]sC>khOST"\\[V8QDa)G#B.3)NQ7DSQ8uVb_0E349o>qjZ-fNAIj(J(%8ME:dDZtJCfFR#uWe7\\Gp@EIXcg$"iSIrHfH^HJ$kqiY!4+8#<jIi.:Ap\'@)o0%Ab[tc`+W)+Shcb^qW>P(D#I\'Mt%rZ!]B03M"&7/9$Z!aq"MXlRP)1[\'7#_H&*@0pkP$ml,uPVH_8j<d[b+U(Ge_Y&[D69#Y(K6E<eR,ufdEKoRRD3fRUboDC!\\W"RcmgnGR!3R.u:?\\eg\'1+S"V:G##qLJrX"aPkfR>b\\`5?PJEmIa7\\fYq51[^QY4!HN~>'



from .pdf import PdfFileReader, PdfFileWriter
from .merger import PdfFileMerger
from .pagerange import PageRange, parse_filename_page_ranges
from ._version import __version__
__all__ = ["pdf", "PdfFileMerger"]



from __future__ import unicode_literals
import datetime

from rlextra.thirdparty import xlrd

def xlCellSpecToCoord(cs):
    """
>>> xlCellSpecToCoord("AA0")    # doctest: +ELLIPSIS
Traceback (most recent call last):
...
ValueError: bad Excel cell ...'AA0'
>>> xlCellSpecToCoord("AA1")
(26, 0)
>>> xlCellSpecToCoord("A1")
(0, 0)
>>> xlCellSpecToCoord("A0")     # doctest: +ELLIPSIS
Traceback (most recent call last):
...
ValueError: bad Excel cell ...'A0'
>>> xlCellSpecToCoord("A9")
(0, 8)
>>> xlCellSpecToCoord("A09")
(0, 8)
>>> xlCellSpecToCoord("A10")
(0, 9)
>>> xlCellSpecToCoord("A255")
(0, 254)
>>> xlCellSpecToCoord("A258")
(0, 257)
>>> xlCellSpecToCoord("IV5")
(255, 4)
>>> xlCellSpecToCoord("IW5")    # doctest: +ELLIPSIS
Traceback (most recent call last):
...
ValueError: bad Excel cell ...'IW5'
>>>

    """
    assert type(cs) == type('')
    A = ord("A")
    Z = ord("Z")
    if A <= ord(cs[1]) <= Z:
        if len(cs) < 3:
            raise ValueError('bad Excel cell %r' % cs)
        col = 26*(ord(cs[0])-A+1) + (ord(cs[1])-A)
        row = int(cs[2:])-1
    else:
        if len(cs) < 2:
            raise ValueError('bad Excel cell %r' % cs)
        col = ord(cs[0])-A
        row = int(cs[1:])-1
    if col > 255 or col < 0 or row < 0:
        raise ValueError('bad Excel cell %r' % cs)
    return col, row

def extract(fileName, sheetName,
            rangeSpec=None, rangeName=None,
            maxRows=None, maxCols=None):
    """This extracts the data in the sheet in list-of-list
    formats.  It tries to be modern and return Python booleans
    and dates.
    """
    from rlextra.thirdparty.xlrd.biffh import \
         XL_CELL_EMPTY, XL_CELL_DATE#, XL_CELL_TEXT, XL_CELL_NUMBER, XL_CELL_BOOLEAN, XL_CELL_ERROR

    if rangeName is not None:
        raise NotImplementedError('rangeName not yet implemented')

    book = xlrd.open_workbook(fileName, verbosity=0)
    sheet = book.sheet_by_name(sheetName)

    if rangeSpec is not None:
        if rangeSpec.count(':') != 1:
            raise ValueError("Bad Excel cell range: %r" % rangeSpec)
        cell1, cell2 = [xlCellSpecToCoord(cellSpec) for cellSpec in rangeSpec.split(":")]
        cols = cell1[0], cell2[0]
        rows = cell1[1], cell2[1]
        colMin, colMax = min(cols), max(cols)+1
        rowMin, rowMax = min(rows), max(rows)+1

    if rangeSpec is None and rangeName is None:
        rowMin = colMin = 0
        rowMax = sheet.nrows+1
        colMax = sheet.ncols+1

    if maxCols is not None:
        colMax = min(colMax, maxCols)
    if maxRows is not None:
        rowMax = min(rowMax, maxRows)

    out = []
    for rowx in range(rowMin, rowMax):
        row = []
        for colx in range(colMin, colMax):
            cellType = sheet.cell_type(rowx, colx)
            cellValue = sheet.cell_value(rowx, colx)

            if cellType is XL_CELL_EMPTY:
                cellValue = None
            elif cellType is XL_CELL_DATE:
                #convert
                yyyy,mm,dd,hh,nn,ss = xlrd.xldate_as_tuple(cellValue, book.datemode)
                cellValue = datetime.date(yyyy,mm,dd)
            else:
                pass #native value is OK

            row.append(cellValue)
        #now trim empties off the row - we don't want extra stuff on the
        #right, which might be hundreds of rows.
        
        while row and row[-1] is None:
            row.pop()

        out.append(row)

    #now trim off any blank rows at the end
    while out[-1:] == []:
        out.pop()
    return out

def _test(filename, sheetName):
    print('running on spreadsheet "%s"' % filename)

    data = extract(filename, sheetName, rangeSpec="A1:B3")#,maxRows=6)
    from pprint import pprint as pp
    pp(data)

def _doctest():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _doctest()
    #_test("test/exceldata.xls", "Sheet1")
